//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------

#ifndef( MA_Skyfakes_Temp)
	#declare MA_Skyfakes_Temp = version;
	#version 3.7;

	//Here we're still inside an If-Block !! (the first #ifndef)

//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------


/* ****************************************************************************************************************************
--------------------------------------------------------
MA_Skyfakes
--------------------------------------------------------
Purpose:		Present some easy and more or less fast to render planets and other objects of the sky.
Author:			Martin Seydler
Website:		www.maetes.com
Documentation:	https://www.maetes.com/en/povray-os/ma_skyfakes
Date:			November 2018 an later
Version:		1.0 (December 2024)
Copyright:		See above (author), below near any macro (orignal author and sources) and everywhere (www.povray.org)
License:		Use it in your projects as you like. A mention and a link to my website would be nice.


General Idea:
------------------
This script generates some easy and simple to render Sky-Stuff.
Maybe not the most realistic, some planets only look nice with a few 100 px or less in size.
But that should be enough to place them somewhere near your primary objects.


Beware:
----------------
All examples are optimized for a camera-angle of 60 to 90 deg and camera-type ultrawideangle.
The size of 1 povray-unit for the planetary body, rings appear bigger.
Especially atmospheres and bozo-patterns react very strange if you scale them,
so prepare yourself to get unfriendly results and adjust some variables.
Sometimes I will mention what values are nice for other scale-factors.


Feel free to contact me if you have found or self-created (really) nice and easy to render ...
------------------------------------------------------------------------------------------------
- planets, moons
- irregulare asteroids
- comets
- nebulaes, galaxies
- or whatelse there above can be seen



Some notes for me, maybe other planets
------------------------------------------
http://www.hoboes.com/NetLife/POV_Simple/POV_Simple_HTML/

Ring-Planet
http://lib.povray.org/searchcollection/index2.php?objectName=NXPlanets&version=1.1&contributorTag=Nekar%20Xenos

Maybe some nice textures, in the lower part of the site
http://lib.povray.org/searchcollection/index2.php

Maybe for another planet/moon:
http://www.f-lohmueller.de/pov_tut/tex/tex_880e.htm

Texture-informations
http://www.f-lohmueller.de/pov_tut/tex/tex_830e.htm


Sehr gute Maps für Planet-Earth:
https://www.blenderguru.com/tutorials/earth-cycles

*/// **************************************************************************************************************************


#ifndef(SF_StarsBGColor)	#declare SF_StarsBGColor = <0,0,0.003>; #end


#ifndef(Colors_Inc_Temp)	#include "colors.inc"		#end


//Dont forget to include ma_skyfakes_v1_texures.inc





/* ****************************************************************************************************************************
SF_BackgroundStars_Pigment

Paints countless background-stars onto the skysphere

Source:
	http://www.hoboes.com/NetLife/POV_Simple/5/2/
	http://wiki.povray.org/content/Reference:Sky_Sphere
	
Example:
	sky_sphere {	SF_BackgroundStars_Pigment (2,5)		}
	
Params:
	Density		Number of stars, use values between 0 and 10 (1 default). 
	Size		Size/brightness of the biggest stars, use values between 0 and 10 (1 default). 
	The parameters affects each other, so combine both to find a good visual appearance.
	Test them with the final resolution of your pic/ani, because the result can varies very much.

Return:
	Pigment:	Fill it into a sky-sphere (dont use as a object)
*/// **************************************************************************************************************************
#macro SF_BackgroundStars_Pigment (Density, Size)
	#local mResult = pigment {
		bozo
		color_map {
			[0.0 color <1,1,1>*3]
			[0.2 + (0.01*Density) color SF_StarsBGColor]
			[1.0 color SF_StarsBGColor]
		}
		scale .0015 * Size/2
	}
	pigment { mResult }
#end
	


	
	

/* ****************************************************************************************************************************
SF_BackgroundSun_Pigment

Paints a sun onto the skysphere
This does not affect the vector of the real-lightsource!
But its much faster to render than an object with a halo/media around it.

Sources:
	http://wiki.povray.org/content/Reference:Sky_Sphere

Example:
	sky_sphere {
		SF_BackgroundSun_Pigment (1,1,-20,30) //Creates a sun near the end of the positive z-axis (littlebit up/left of it).
	}
	
Params:
	BodyScale	scales the size of the suns body (1 uses the untouched, realistic value, > 1 means bigger body)
	HaloScale	scales the size of the surrounding halo (1 = untouched, < 1 smaller, > 1 bigger, as above)
	RotateX		Float/Degrees (-90 to 90)  , moves sun down (-) or up (+)
	RotateY		Float/Degrees (-180 to 180), moves sun to the left (-) or right (+)

Return:
	Pigment:	Fill it into a sky-sphere (dont use as a object)
*/// **************************************************************************************************************************
#macro SF_BackgroundSun_Pigment(BodyScale, HaloScale, RotateX, RotateY)

	//Visible Diameter of the Sun is about 0.52 degrees, divided by 2 for Radius and another 2 for Scale-Factor (see below, Sun-Pigment)
	#local SunRadVis	= 1/360*0.52/2/2;					//Original radius of the sun
	#local SunBody		= BodyScale * SunRadVis / 2;		//Radius of the visible, solid Sun-Body 
	#local SunHalo		= HaloScale * SunRadVis * 20 / 2;	//Radius of the glowing halo around the Sun

	#local mResult = pigment {
		gradient -z
		color_map {
			[0.00	 SunBody	color rgb <1.0, 0.2, 0.0> * 6				color rgb <1.0, 0.2, 0.0> * 1.5	]
			[SunBody SunHalo	color rgbt <0.6, 0.08, 0.0, 0.8>			color rgbt <0.0, 0.0, 0.0, 1>	]
			[SunHalo 1			color rgbt <0.0, 0.0, 0.0, 1>				color rgbt <0.0, 0.0, 0.0, 1>	]
		}
		scale 2						//Important, otherwise at 90 Degrees something like a Halo or Horizon will be visible.
		translate -1				//Dont know what this does, BUT ITS IMPORTANT too.
		rotate <RotateX, RotateY, 0>
	}
	pigment { mResult }
#end






/* ****************************************************************************************************************************
*******************************************************************************************************************************
SF_Gasgiant1

Creates jupiter-like planet.
Borrowed from Bob H. - see http://objects.povworld.org/cat/Space/

Example:
	object { SF_Gasgiant1 ("", "", "") }
	
Params:
	mParentName		All first 3 Parameter are maetes-standard, let them = "" if you dont know what to do with it.
	mNewName
	mParams

Return:
	Object
*******************************************************************************************************************************
*/// **************************************************************************************************************************

#macro SF_Gasgiant1 (mParentName, mNewName, mParams)
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);		#end

	#local mResult = sphere {
		0,1
		texture {
			pigment {
				gradient y turbulence .05 frequency -1
				color_map {
					[0 color rgb <.85,.8,.725>] // poles
					[.05 color rgb <.875,.875,.775>]
					[.15 color rgb <.925,.875,.825>]
					[.25 color rgb <.9,.875,.85>]
					[.275 color rgb <.825,.8,.775>]
					[.33 color rgb <.925,.85,.875>]
					[.45 color rgb <.975,.95,.925>]
					[.54 color rgb <.85,.6,.75>]
					[.55 color rgb <.95,.55,.4>] // Great Red Spot
					[.56 color rgb <.9,.95,.85>]
					[.6 color rgb <.85,.8,.875>]
					[.65 color rgb <.8,.45,.55>]
					[.725 color rgb <.725,.55,.4>] // dark bands
					[.8 color rgb <.725,.575,.425>]
					[.9 color rgb <.85,.7,.725>]
					[.95 color rgb <.75,.725,.775>]
					[1 color rgb <.95,.8,.75>] // equator
				}
				scale		<.2,1,.2>
				rotate		15*y
				translate	-.05*y
				warp {
					black_hole <.5,-.5,-.8>,.25 falloff .3 strength 1.2 turbulence 0 inverse
				}
			}
			
			finish {
				//ambient .03 //Use defaults at first.
				diffuse .55 
				phong .075 phong_size 1.5 specular .025 roughness .01
			}
		}
		#ifdef( MA_Screenplay_Temp )
			rotate SCR_GetVector (mMyName, "", "rotate", <0,0,0>)
		#end
	}
	
	
	//Add the atmosphere
	#if (MA_GetParam_S(mParams, "atmo-type", "") != "")
		#local mResult = SF_AddAtmo(mResult, mParams);
	#end
	//Add ringsystem
	#if (MA_GetParam_S(mParams, "rings-type", "") != "")
		#local mResult = SF_AddRings(mResult, mParams);
	#end
	
	mResult
#end


/* ****************************************************************************************************************************
*******************************************************************************************************************************
SF_Moon1

Creates a moon like (like!) that of ours.

Example:
	object { SF_Moon1 ("", "", "") }

Params:
	mParentName		All three are maetes-standard, let them = "" if you dont know what to do with it.
	mNewName
	mParams
	
Return:
	Object
*******************************************************************************************************************************
*/// **************************************************************************************************************************

#macro SF_Moon1 (mParentName, mNewName, mParams)
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);		#end

	#local mResult = sphere {
		0,1
		texture {
			bozo
			//gradient x
			turbulence 0.5
			texture_map {
				[0.00	SF_OT_Moon3 ]
				[0.10	SF_OT_Moon1 ]
				[0.30	SF_OT_Moon1 ]
				[0.50	SF_OT_Moon2 ]
				[0.60	SF_OT_Moon2 ]
				[0.70	SF_OT_Moon3 ]
				[1.00	SF_OT_Moon3 ]
			}
			rotate <0,0,0>
		}
		#ifdef( MA_Screenplay_Temp )
			rotate SCR_GetVector (mMyName, "", "rotate", <0,0,0>)
		#end
	}
	
	//Add the atmosphere
	#if (MA_GetParam_S(mParams, "atmo-type", "") != "")
		#local mResult = SF_AddAtmo(mResult, mParams);
	#end
	//Add ringsystem
	#if (MA_GetParam_S(mParams, "rings-type", "") != "")
		#local mResult = SF_AddRings(mResult, mParams);
	#end
	
	mResult
#end




/* ****************************************************************************************************************************
Creates a Class-M planet

See: https://en.wikipedia.org/wiki/Star_Trek_planet_classification

Examples:
	object { SF_Minshara1 ("", "", "") }
	object { SF_Minshara1 ("", "", "atmo-type: simple-1;") }

Params:
	mParentName		All three are maetes-standard, let them = "" if you dont know what to do with it.
	mNewName
	mParams

Return:
	Object
*/// **************************************************************************************************************************
#macro SF_Minshara1 (mParentName, mNewName, mParams)
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);		#end

	#local mResult = sphere {
		0,1
		texture {
			pigment {
				bozo turbulence 3.86
				color_map {
					[0.0 color Brown*0.5]
					[0.35 color rgb <0.2, 0.5, 0.2>*0.5]
					[0.45 color rgb <0.5, 0.5, 1.0>]
					[0.65 color rgb <1.0, 1.0, 1.0>]
					[1.0 color rgb <0.5, 0.5, 0.5>]
				}
				frequency 0.75
			}
			finish { ambient 0.007 diffuse 1.1 phong 0.5 }
			//finish {ambient .03 diffuse .55 phong .075 phong_size 1.5 specular .025 roughness .01}
			scale 1.3
			
			#ifdef( MA_Screenplay_Temp )
				rotate SCR_GetVector (mMyName, "", "rotate", <0,0,0>)
			#end
		}
	}

	//Add the atmosphere
	#if (MA_GetParam_S(mParams, "atmo-type", "") != "")
		#local mResult = SF_AddAtmo(mResult, mParams);
	#end
	//Add ringsystem
	#if (MA_GetParam_S(mParams, "rings-type", "") != "")
		#local mResult = SF_AddRings(mResult, mParams);
	#end
	
	mResult
#end

	

/* ****************************************************************************************************************************
Adds an atmosphere to a planet

This is usable but very under construction, because I have no clue how to create good atmospheres.

The atmo "simple-1" is optimized for a planet 1 pu radius.
The farther away (and maybe the bigger? Damn) the planet is, the thinner the atmosphere will become.
Best way to modify it is to use the param "atmo-scattering-factor".
As bigger the distance as smaller this value has to be.
Maybe 1000 pixel distance can lead to a scattering-factor of 0.005 or so, instead of the default = 5 (at the moment).

Params:
	xResult		a planet as an object, 1 pu radius (you can resize it later)
	mParams		the params
Return:
	Object
*/// **************************************************************************************************************************

#macro SF_AddAtmo(xResult, mParams)

	#local mDefaultColor		= MA_GetParam_V(mParams, "atmo-default-color", <1,0.9,0.9,0.97>);		
	#local mType				= MA_GetParam_S(mParams, "atmo-type", "");

	//Note: not all of the different types will use this parameters.
	//Check the code if they do.
	//All parameters are optimized do display planets of a few 100 pixel.
	#local mRadius				= MA_GetParam_F(mParams, "atmo-radius",				1);					//Radius of the object that has to get the atmosphere, 1 most common.
	#local mRadiusInner 		= MA_GetParam_F(mParams, "atmo-radius-inner",		1.001)	* mRadius;
	#local mRadiusOuter 		= MA_GetParam_F(mParams, "atmo-radius-outer",		1.0165)	* mRadius;
	#local mScatteringTyp		= MA_GetParam_F(mParams, "atmo-scattering-type",		1);
	#local mScatteringColor		= MA_GetParam_V(mParams, "atmo-scattering-color",	<0.1,0.11,0.5>);	//Blue, Earth
	
	//#local mScatteringFactor	= MA_GetParam_F(mParams, "atmo-scattering-factor",	0.005);		//For distant, big objects
	#local mScatteringFactor	= MA_GetParam_F(mParams, "atmo-scattering-factor",	5);			//For near, small objects
	
	#local mEmission			= MA_GetParam_F(mParams, "atmo-emission",			0.000002);			
	
	//This was/is a try to respect the different sizes of the object (scaling/distancing the object can end up in very unexpected results)
	//Will work on it later.
	//#local mEmissionRadius		= pow(mRadius, 4);
	//#local mEmissionRadius	= 1*sqrt(3)/vlength(mRadius);
	#local mEmissionRadius		= mRadius;

	#local mAtmo = difference {
	
		//Creates a thin hollow sphere that contains the atmosphere
		sphere	{ 0, mRadiusOuter }
		sphere	{ 0, mRadiusInner }
		hollow
	
		#if (mType = "simple-1")
			pigment	{ rgbf 1 }
			interior {
				media{
					scattering {
						mScatteringTyp,
						mScatteringColor * mScatteringFactor
					}
					//scattering{2,<0.001,0.0005,0.0005>*.5} //Mars, red
					intervals 1
					emission mEmission / mEmissionRadius // 0.005 or smaller
				}
			}
		#elseif (mType = "")
			//DoNothingLoop
		#else
			//Nothing to do at the Moment
			MA_Terminal_S("SF_AddAtmo - No valid atmo-type choosen", mType)
			pigment{ color rgbf mDefaultColor}		//Color if no valid type was choosen (slightly pink here)
		#end
	}
	
	//Union object/planet and atmosphere
	#local mResult	= union {
		object { xResult }
		object { mAtmo }
	}
	
	mResult
#end


/* ****************************************************************************************************************************
Adds a ringsystem to the planet

Attention: the rings are discs, so seen edge-on you cant see them.
Rotate the resulting object/planet a littlebit at all vectors or move the camera Y+ to find them.

Return:
	Object
*/// **************************************************************************************************************************

#macro SF_AddRings(xResult, mParams)

	#local mType	= MA_GetParam_S(mParams, "rings-type", "");
	#local mResult	= xResult;
	
	#if (mType = "saturn-1")
		//Saturn-Ringsystem, here in thousands km, so divide by the Divisor (see disc{...} )
		//Its a copy-paste from another project and I refused to recalculate the values by hand.
		#local mDivisor = 40;
		#local mRingData = array[6][7] {
			{  74.500,  92.000, 0.90, 0.88, 0.79, 0.6, 0.4 }	//C ring (innermost, more translucent) 
			{  92.000,  98.390, 0.99, 0.88, 0.79, 0.2, 0.4 } 	//Inner B ring (translucent) 
			{  98.390, 117.500, 0.99, 0.88, 0.79, 0.1, 0.4 }	//Outer B ring (solid, inside Cassini Division) 
			{ 122.200, 133.570, 0.99, 0.88, 0.79, 0.1, 0.4 }	//Inner A ring (between Cassini and Encke divisions) 
			{ 133.895, 136.800, 0.99, 0.88, 0.79, 0.2, 0.4 }	//Outer A ring (outside Encke division) 
			{ 140.210, 140.460, 0.99, 0.88, 0.79, 0.1, 0.1 } 	//F ring (solid, outermost thin ring) 
		}

		#local mRingSystem = union {
			#for (i, 0, dimension_size(mRingData,1) - 1)
				disc { 
					<0, 0, 0>	y,
					mRingData[i][1] / mDivisor,
					mRingData[i][0] / mDivisor
					pigment {color rgbt <mRingData[i][2], mRingData[i][3], mRingData[i][4], mRingData[i][5]>} 
					finish {ambient 0.0015 diffuse mRingData[i][6]} 
					double_illuminate	//Underside (as seen from sun) also illuminated
				}
			#end
		}
		#local mResult = union {
			object { mResult } 
			object {
				mRingSystem
				//rotate <10,10,10> //For tests only
			}
		}
	#elseif (mType = "")
		//DoNothingLoop()
	#else
		//Nothing to do at the Moment
		MA_Terminal_S("SF_AddRings - No valid rings-type choosen", mType)
	#end
	
	mResult
	
#end



/* ****************************************************************************************************************************
SF_Horizon1 ()

Creates a simple horizon.
Colors: blue, green, brown, white clouds.

Links:
	http://www.povray.org/documentation/view/3.7.0/77/
	https://duckduckgo.com/?q=povray+planet+earth&t=ffsb&ia=web

Example:
	object { SF_Horizon1 ( 10000, 100, "b", <0,0,0>, 1) }
	
This object waits to become screenplay-compatible.
Maybe next version.
	
Params:
	diameter		Diameter of the planet
	height			Height above Surface (ISS = 300 km)
	clouds-layers	0 = no clouds, 1 or 2 for the number of layers
	
Parameters:
	mParentName, mNewName, mParams as always (leave blank if you dont know what to do)
	mPosition		where the horizon has to appear (left, right, top, bottom, also l,r,t,b allowed).
	mRotation		rotation-vector, to look for nicer scenes or to let rotate it in animations
	
Return:
	Object
*/// **************************************************************************************************************************
#macro SF_Horizon1 (mParentName, mNewName, mParams, mPosition, mRotation)
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);		#end

	#local mDia				= MA_GetParam_F(mParams, "diameter",		5000);	//Diameter of the planet
	#local mHeight			= MA_GetParam_F(mParams, "height",	 		100);	//Height above Surface (ISS = 300 km)
	#local mCloudsLayers	= MA_GetParam_F(mParams, "clouds-layers",	2);		//0 = no clouds, 1 or 2 for the number of layers

	//Dia and Height given in km, so multiply by 1000
	#local mPlanetRadius	= mDia / 2 * 1000; 
	#local mHeight			= mHeight * 1000;
	#local mDistance		= mPlanetRadius + mHeight * 1;
	
	#if		(mPosition = "l" | mPosition = "left")		#local mTransVector	= < -mDistance, 0, 0>;
	#elseif (mPosition = "r" | mPosition = "right")		#local mTransVector	= < +mDistance, 0, 0>;
	#elseif (mPosition = "t" | mPosition = "top")		#local mTransVector	= < 0, +mDistance, 0>;
	#else												#local mTransVector	= < 0, -mDistance, 0>;
	#end

	#local mResult = object {
		sphere {
			<0,0,0>, mPlanetRadius
			texture { SF_Horizon1_LandArea				scale mPlanetRadius/10 }
			texture { SF_Horizon1_OceanArea			scale mPlanetRadius/12 }
			#if (mCloudsLayers >= 1)	texture { SF_Horizon1_CloudArea3			scale mPlanetRadius/12 } 	#end
			#if (mCloudsLayers >= 2)	texture { pigment { SF_Horizon1_Clouds1 }	scale mPlanetRadius/12 }	#end
		}
		rotate mRotation
		#ifdef( MA_Screenplay_Temp )	rotate SCR_GetVector (mMyName, "", "rotate", <0,0,0>)		#end
		
		translate mTransVector
	}
	
	mResult
#end







//-----------------------------------------------------------------------------------------------------------------------------
//This is the end of the very first If (that checks if this file was included before)
//Never delete it, or else ...
	#version MA_Skyfakes_Temp;
#end
//-----------------------------------------------------------------------------------------------------------------------------
