//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------
#ifndef( MA_Tussi_Temp)
	#declare MA_Tussi_Temp = version;	//raises a parser-warning if the file does not gets included. Ignore it.
	#version 3.7;
	//Here we're still inside an If-Block !! (the first #ifndef)
//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------


/* ****************************************************************************************************************************

--------------------------------------------------------
MA_Tussi
--------------------------------------------------------
Purpose:		Show a simple, small, fast to render Spacecraft
Author:			Martin Seydler
Website:		www.maetes.com
Documentation:	https://www.maetes.com/en/povray-os/ma_tussi
Date:			2018 an later
Version:		1.0 (December 2024)
Copyright:		See above (author)
License:		Use it in your projects as you like. A mention and a link to my website would be nice.


Helpful Command-Lines (change the path):
------------------------------------------
povray +Ima_tussi.inc +Otest/ma_tussi.png +w800 +h600 -GR +A
povray +Ima_tussi.inc +Otest/ma_tussi.png +w1200 +h800 +GD +GF -GR +GS +GW +Q9 +A


To open the resulting picture in a Browser (change the path):
----------------------------------------------------------------
file:///home/pi/Projekte/povray/packages_ma/ma_tussi/test/



*/// **************************************************************************************************************************


#ifndef(Colors_Inc_Temp)	#include "colors.inc"		#end
#ifndef(Textures_Inc_Temp)	#include "textures.inc"		#end




//-------------------------------
//Private variables
//only for internal use
//-------------------------------
#declare TU_Private_LegsY_Real	= 0;
#declare TU_Private_LegsY_Max	= 0;


//-------------------------------
//Texture Main Body
//-------------------------------
#ifndef(TU_Spacecraft_Texture)
	#declare TU_Spacecraft_Texture = texture {
		pigment{
			marble scale 0.01 turbulence 0.0
			color_map { 
				//[0.0 color rgb <0.08, 0.08, 0.08>]
				//[1.0 color rgb <0.0 , 0.0 , 0.0 >]
				[0.0 color rgb <0.9, 0.9, 0.9>]
				[1.0 color rgb <0.0 , 0.0 , 0.0 >]
			}
			scale 2
			//http://www.f-lohmueller.de/pov_tut/tex/tex_780d.htm
			warp { cylindrical
				orientation <0,1,0>
				dist_exp 0
			} 
		}
	}
	#if (0)
		#declare TU_Spacecraft_Texture = texture {
			pigment{ rgbf < 0.7, 0.7, 0.7, 0.9> }
			finish { reflection 0.1 phong 1}
		}
	#end
	
#end
//-------------------------------
//Texture Plates front/aft end
//-------------------------------
#ifndef(TU_Spacecraft_Plates_Texture)
	#declare TU_Spacecraft_Plates_Texture =	texture {
		pigment { color Gray15 }
		//pigment { color Red }
		//normal	{ bumps 0.1 scale 0.015 }
		//finish { reflection 1 phong 1}
	}
#end

//-------------------------------
//Small thrusters
//-------------------------------
#ifndef(TU_Thruster_Box_Color)		#declare TU_Thruster_Box_Color		=		color Yellow;				#end
#ifndef(TU_Thruster_Nozzle_Color)	#declare TU_Thruster_Nozzle_Color	=		color Red;					#end
#ifndef(TU_ThrusterBox_Plate_Color)	#declare TU_ThrusterBox_Plate_Color	=		color White;				#end
#ifndef(TU_Thruster_Flame_Texture)
	#declare TU_Thruster_Flame_Texture = texture {
		//pigment { rgbf < 0.9, 0.5, 0.5, 0.9> }
		pigment { rgbt < 0.9, 0.2, 0.2, 0.4> }

	}
#end

//-------------------------------
//Main engine at the aft end
//-------------------------------
#ifndef(TU_Engine_Box_Color)		#declare TU_Engine_Box_Color		=		color Yellow;				#end
#ifndef(TU_Engine_Nozzle_Texture)
	#declare TU_Engine_Nozzle_Texture = texture {
		pigment {
			marble scale 0.03 turbulence 0.0
			color_map{ 
				[0.0 color rgb <0.08, 0.08, 0.08>]
				[1.0 color rgb <0.0 , 0.0 , 0.0 >]
			}
			//http://www.f-lohmueller.de/pov_tut/tex/tex_780d.htm
			warp { cylindrical
				orientation <1,0,0>
				dist_exp 0
			} 
		}
	}
#end
#ifndef(TU_Engine_Flame_Texture)
	#declare TU_Engine_Flame_Texture = texture {
		#if (1)
			//Steady flame
			//pigment{ rgbf < 0.8, 0.2, 0.8, 0.7> }
			//pigment{ rgbf < 0.8, 0.8, 0.2, 0.98> }
			pigment{ rgbf < 0.9, 0.4, 0.1, 0.9> }
			
		#elseif	(1)
		
		#else
			//Testbed for variable transparency (19.5.2019 ff)
			pigment {
				//rgbf < 0.9, 0.3, 0.9>
				gradient <0,1,0> //frequency 0.88 
				color_map { 
					[0.0 color rgb < 0.9, 0.2, 0.1 > transmit 0.3]
					[0.3 color rgb < 0.9, 0.2, 0.1 > transmit 0.7]
					[0.8 color rgb < 0.9, 0.2, 0.1 > transmit 1.0]
					[1.0 color rgb < 0.9, 0.2, 0.1 > transmit 1.0]
				}
				scale <1,1.1,1>
				translate <0,0.05,0>
			}
		#end
	}
#end


//-------------------------------
//Landinglegs
//-------------------------------
#ifndef(TU_Legs_Color)				#declare TU_Legs_Color			=		color Cyan * 0.2;		#end
#ifndef(TU_Foot_Color)				#declare TU_Feet_Color			=		color Orange*0.5;		#end


//-------------------------------
//Booster
//-------------------------------
#ifndef(TU_Booster_ColorMain)		#declare TU_Booster_ColorMain	=		color White;			#end
#ifndef(TU_Booster_ColorStrips)		#declare TU_Booster_ColorStrips	=		color Gray10;			#end
#ifndef(TU_Booster_ColorCons)		#declare TU_Booster_ColorCons	=		color Blue;		#end	//Connector
#ifndef(TU_Booster_Nozzle_Texture)	#declare TU_Booster_Nozzle_Texture =
	texture {
		pigment{ marble scale 0.13 turbulence 0.0
			color_map{ 
				[0.0 color rgb <0.08, 0.08, 0.08> * 5]
				[1.0 color rgb <0.0 , 0.0 , 0.0 > * 5]
			}
			//http://www.f-lohmueller.de/pov_tut/tex/tex_780d.htm
			warp { cylindrical
				orientation <1,0,0>
				dist_exp 0
			} 
		}
	}
#end

//http://www.f-lohmueller.de/pov_tut/tex/tex_755e.htm
#ifndef(TU_Booster_Flame_Texture)
	#declare TU_Booster_Flame_Texture = texture {
		pigment {
		#if (1)
			rgb < 0.5, 0.5, 0.2> * 2 transmit 0.6
		#else
			gradient y
			color_map{ 
			#if (0)
				[1.0 color rgb < 3.5, 0.5, 0.2> * 1 transmit 0.0]
				[0.9 color rgb < 0.5, 0.5, 0.2> * 2 transmit 0.0]
				[0.5 color rgb < 0.5, 0.5, 0.2> * 2 transmit 0.5]
				[0.0 color rgb < 0.5, 0.5, 0.2> * 2 transmit 0.0]
			#else
				[0.0 color rgb < 0.5, 0.5, 0.2> * 2 transmit 0.0]
				[0.5 color rgb < 0.5, 0.5, 0.2> * 2 transmit 0.5]
				[0.9 color rgb < 0.5, 0.5, 0.2> * 2 transmit 0.0]
				[1.0 color rgb < 3.5, 0.5, 0.2> * 1 transmit 0.0]
			#end
			}
		#end
		}
	}
#end





//-------------------------------
//Solarcells
//-------------------------------
#ifndef(TU_Solarcells_Mast_Color)		#declare TU_Solarcells_Mast_Color	= color Grey;			#end
#ifndef(TU_Solarcells_Hinge_Color)		#declare TU_Solarcells_Hinge_Color	= color Gray80;			#end
#ifndef(TU_Solarcells_Border_Color)		#declare TU_Solarcells_Border_Color	= color Gray20;			#end
#ifndef(TU_Solarcells_Cells_Color)		#declare TU_Solarcells_Cells_Color	= color rgb <0,0,0.4>;	#end
#ifndef(TU_Solarcells_Cells_Texture)
	#declare TU_Solarcells_Cells_Texture = texture {
		pigment {
			cells scale 0.12 turbulence 0
			color_map { [0.0 color TU_Solarcells_Cells_Color] [1.0 color rgb <0,0,0.0>] }
		}				
		finish { reflection 0.5 ambient 0.75 diffuse 0.9}
	}
#end

//-------------------------------------------------------------------------
//Parameter
//There is no need to change them here.
//Use third parameter of GetTussi_Base() to do that.
//Both gets concatted inside the macro, by default the last found rules.
//object { MA_Tussi_Base("", "", "body-dia: 10; engine-active:1", "") }
//-------------------------------------------------------------------------
#ifndef(TU_Params_Default)
	#declare TU_Params_Default = concat (
		// --- Main params ---
		";body-dia: 				4",			//Diameter of the body, not included unfolded solarwings, thusters, thruster-flames
		";body-height:				6",			//Height of the spacecraft-body, not included the impactor at the top, engine and engine-flame
		// --- Engine (at the aft-end) ---
		";engine-height:			1.5",		//Engine-height (box and nozzle together)
		";engine-dia:				1.3",		//Max diameter of the engine-nozzle
		";engine-flame-height:		3",			//Factor, gets multiplied with engine-height
		";engine-flame-dist:		1.1",		//Factor, creates a gap between the nozzle and the flame, with respect to engine-flame-height (1 = no gap, 1.1 a small)
		";engine-active:			0",			//0 = no flame, > 0 = engine active (use flame-heigth to throttle it up and down)

		// --- Payload-Adapter ---
		";pa-dia:					2",			//Diameter Payload-Adapter
		";pa-height:				0",			//Height, 1.5 = full length, 0 = not shown

		// --- Thruster ---
		";thruster-size:			0.5",		//Length of a small thruster. Half of it the box, half the nozzle
		";thruster-dia-1:			0.8",		//Inner/smaller diameter
		";thruster-dia-2:			1.5",		//Outer/bigger diameter
		";thruster-flame-dist:		3",			//distance of flame (in thruster-sizes)
		";thruster-flame-length:	 20",		//Length of flame (in thruster-sizes)
		// --- Solarcells ---
		";sc-offaxis:				45",		//Where the wings are mounted on the cylinders (0 = same as thrusters z-axis, between thrusters)
		";sc-pos-y:					0.30",		//0 = aft-end (near engine), 0.5 center of body, 1 = front-end
		";sc-mast-height:			0.5",		//Height of the slim mast (without cells)
		";sc-wing-height:			3",			//Height of all solarpanels together
		";sc-wing-width:			2",			//Width of the solarcells
		";sc-panels:				3",			//Number of solarcell-panels (not wings!)
		";sc-angle:					0",			//0 = Solarcells folded, 90 = unfolded
		";sc-tilt:					0",			//tilts/rotates the solarcells, 0 => flat on spacecraft-side, + => counterclockwise when seen from with the tip in front
		// --- Landing-Legs ---
		";legs-length:				2",			//Length (in Y-Direction) when fully stretched. Should be higher or equal than engine-height
		";legs-length-factor:		0",			//Factor 0 to 1 to inflate them (0 = inflated, 1 = stretched)
		";legs-length-factor-min:	0.2",		//Minimum-Factor
		";legs-offset-legs:			1.0",		//Offset from the center
		";legs-offset-feet:			2.5",		//Dito, use highter value than above to spread them to the sides
		";legs-radius-legs:			0.06",		//As the name says. 0.02 is very thin.
		";legs-radius-feet:			0.4",		//dito
		";legs-height-feet:			0.1",		//dito
		// --- Booster ---
		";booster:					0",			//0 = no boosters, 1 = two at each side (at the moment)
		";booster-radius:			0.3",
		";booster-height:			4",
		";booster-num-strips:		4",			//separates the booster optically in few parts
		";booster-height-strips:	0.1",
		";booster-length-cons:		0.3",		//Connectors between Tussi and Boosters (also defines the separation between them)
		";booster-height-cons:		0.05",
		";booster-active:			0",			//0 or 1
		";booster-flame-height:		7",
		";booster-flame-radius:		1",
		
		""
	);
#end

	



/* ****************************************************************************************************************************
TU_Tussi()

Creates a simple spacecraft

CenterTypes:
	All types ignores engine- or thruster-flames! Maybe further types will be available in the future)
	[empty]			- "" = as assembled (centered body/cylinder, no matter how long legs/nozzle/boosters are or if impactor is attached)
	standing		- as it would stand with fully deployed legs (no matter if deployed), hardware in y+ EXEPT the nozzle (if legs are shorter than nozzle)
	standing-real	- as "standing" but with respect to param "legs-length-factor" (not recommended)
	
Params:
	mParentName	
	mNewName	
	xParams
	mCenterType		
Return:
	Object
*/// **************************************************************************************************************************
#macro TU_Tussi (mParentName, mNewName, xParams, mCenterType)

	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);		#end
	#ifndef( MA_Logfile_Temp )		#local mFunc = ""; #else #local mFunc = LOG_MacroHeader("TU_Tussi");	#end


	#local mParams			= concat (TU_Params_Default, ";", xParams);

	#local mRadius			= MA_GetParam_F(mParams, "body-dia", 2) / 2;
	#local mHeight			= MA_GetParam_F(mParams, "body-height", 5);
	#local mThrusterSize	= MA_GetParam_F(mParams, "thruster-size", 0.5);
	#local mSolarWingAngle	= MA_GetParam_F(mParams, "sc-angle", 0);
	#local mSolarWingTilt	= MA_GetParam_F(mParams, "sc-tilt", 0);

	#local mBooster			= MA_GetParam_F(mParams, "booster", 1);

	#local mPADia			= MA_GetParam_F(mParams, "pa-dia", 0);
	#local mPAHeight		= MA_GetParam_F(mParams, "pa-height", 0);

	
	// -------------------------------
	// Main body of the spacecraft
	// -------------------------------
	#local mCan = union {
		//Plates at front and end
		cylinder { <0,-mHeight/2 - 0.001,0>,	<0,-mHeight/2 + 0.1,0>, mRadius * 1.01  texture { TU_Spacecraft_Plates_Texture	} }
		cylinder { <0, mHeight/2  + 0.001,0>,	<0, mHeight/2 - 0.1,0>, mRadius * 1.01	texture { TU_Spacecraft_Plates_Texture	} }
			
		//Main body
		cylinder { <0,-mHeight/2,0>,			<0,mHeight/2,0>,		mRadius			texture { TU_Spacecraft_Texture			} }
	}
	
	// -------------------------------
	// Small thrusters at all 4 edges
	// At the top and the bottom
	// tb = short for thruster-box
	// f = for(ward) boxes (flight-direction)
	// a = aft (near the mainengines)
	// l/b/r/t = left/top/right/bottom
	// -------------------------------
	#local mThrusterBox_t1	= TU_ThrusterBoxFor		( mMyName, "tb-fl", mParams)
	#local mThrusterBox_t2	= TU_ThrusterBoxFor		( mMyName, "tb-fb", mParams)
	#local mThrusterBox_t3	= TU_ThrusterBoxFor		( mMyName, "tb-fr", mParams)
	#local mThrusterBox_t4	= TU_ThrusterBoxFor		( mMyName, "tb-ft", mParams)
	#local mThrusterBox_b1	= TU_ThrusterBoxBack	( mMyName, "tb-al", mParams)
	#local mThrusterBox_b2	= TU_ThrusterBoxBack	( mMyName, "tb-ab", mParams)
	#local mThrusterBox_b3	= TU_ThrusterBoxBack	( mMyName, "tb-ar", mParams)
	#local mThrusterBox_b4	= TU_ThrusterBoxBack	( mMyName, "tb-at", mParams)
	
	// -------------------------------
	// Landing-Legs
	// -------------------------------
	#local mLeg1		= TU_LandingLegs	( mMyName, "leg1", mParams);
	#local mLeg2		= TU_LandingLegs	( mMyName, "leg2", mParams);
	#local mLeg3		= TU_LandingLegs	( mMyName, "leg3", mParams);
	#local mLeg4		= TU_LandingLegs	( mMyName, "leg4", mParams);

	// -------------------------------
	// Booster
	// -------------------------------
	#if (mBooster = 1)
		#local mBooster1	= TU_Booster		( mMyName, "booster1", mParams)
		#local mBooster2	= TU_Booster		( mMyName, "booster2", mParams)
		#local mBooster3	= TU_Booster		( mMyName, "booster3", mParams)
		#local mBooster4	= TU_Booster		( mMyName, "booster4", mParams)
	#end
	
	// -------------------------------
	// Main-Engine at the lower end
	// -------------------------------
	#local mEngine			= TU_MainEngine			( mMyName, "engine", mParams)


	// -------------------------------
	// Payload-Adapter (if any)
	// -------------------------------
	#if (mPAHeight > 0)
		#local mPA 			= TU_PayloadAdapter	(mPADia, mPAHeight)
	#end
	

	// -------------------------------
	// Solar-Wings
	// -------------------------------
	#ifdef( MA_Screenplay_Temp )
		#local mSolarWingAngle	= SCR_GetNumber (mMyName, "", "sc-angle", mSolarWingAngle);
		#local mSolarWingTilt	= SCR_GetNumber (mMyName, "", "sc-tilt", mSolarWingTilt);

	#end
	#local mSolarWingAngle	= mSolarWingAngle * -1;
	#local mSolarWingTilt	= mSolarWingTilt + 90;
	
	#local mSolarWing1		= object { TU_SolarWing		( mMyName, "wing-1", mParams ) rotate <0, -mSolarWingTilt, mSolarWingAngle> translate <mRadius, -0, 0> rotate <0,0,0>} //First z from 0 to -90 to unfold
	#local mSolarWing2		= object { TU_SolarWing		( mMyName, "wing-1", mParams ) rotate <0, +mSolarWingTilt, mSolarWingAngle> translate <mRadius, -0, 0> rotate <0,0,0>} //First z from 0 to -90 to unfold

	// -------------------------------
	// Stitch all of them together
	// -------------------------------
	#if (1)
		#local mResult = union {
			object { mCan }
			object { mThrusterBox_t1		rotate <0,-90, 0>	translate < 0, mHeight/2 - mThrusterSize/2, mRadius>	rotate <0,  0,0> }
			object { mThrusterBox_t2		rotate <0,-90, 0>	translate < 0, mHeight/2 - mThrusterSize/2, mRadius>	rotate <0, 90,0> }
			object { mThrusterBox_t3		rotate <0,-90, 0>	translate < 0, mHeight/2 - mThrusterSize/2, mRadius>	rotate <0,180,0> }
			object { mThrusterBox_t4		rotate <0,-90, 0>	translate < 0, mHeight/2 - mThrusterSize/2, mRadius>	rotate <0,270,0> }
			object { mThrusterBox_b1		rotate <0,-90, 0>	translate < 0,-mHeight/2 + mThrusterSize/2, mRadius>	rotate <0,  0,0> }
			object { mThrusterBox_b2		rotate <0,-90, 0>	translate < 0,-mHeight/2 + mThrusterSize/2, mRadius>	rotate <0, 90,0> }
			object { mThrusterBox_b3		rotate <0,-90, 0>	translate < 0,-mHeight/2 + mThrusterSize/2, mRadius>	rotate <0,180,0> }
			object { mThrusterBox_b4		rotate <0,-90, 0>	translate < 0,-mHeight/2 + mThrusterSize/2, mRadius>	rotate <0,270,0> }
			
			object { mEngine				rotate <180,0, 0>	translate < 0,-mHeight/2, 0>	rotate <0,0,0> }
			
			#ifdef(mPA)
				//object { mPA				translate < 0,-mHeight/2, 0> }
				object { mPA				translate < 0,-mHeight/2-mPAHeight, 0> }
			#end
			
			#local mR = 0;
			object { mLeg1					translate <0,-mHeight/2, 0>	rotate y*(90*0+mR) }
			object { mLeg2					translate <0,-mHeight/2, 0>	rotate y*(90*1+mR) }
			object { mLeg3					translate <0,-mHeight/2, 0>	rotate y*(90*2+mR) }
			object { mLeg4					translate <0,-mHeight/2, 0>	rotate y*(90*3+mR) }
			
			#local mR = -45;
			#local mD = 10;
			#local mTX = MA_GetParam_F(mParams, "booster-radius",			0.3	) + MA_GetParam_F(mParams, "booster-length-cons",4);
			
			#if (mBooster = 1)
				object { mBooster1				translate <-mRadius-mTX,-mHeight/2, 0>	rotate y*(180*0+mD+mR) }
				object { mBooster2				translate <-mRadius-mTX,-mHeight/2, 0>	rotate y*(180*0-mD+mR) }
				object { mBooster3				translate <-mRadius-mTX,-mHeight/2, 0>	rotate y*(180*1+mD+mR) }
				object { mBooster4				translate <-mRadius-mTX,-mHeight/2, 0>	rotate y*(180*1-mD+mR) }
			#end


			#local mOffAxis	= MA_GetParam_F(mParams, "sc-offaxis",	0);
			#local mPosY	= MA_GetParam_F(mParams, "sc-pos-y",	0);
			#local mPosY	= (mPosY*2 - 1 ) * mHeight/2;
			
			
			object { mSolarWing1			rotate <0,90-mOffAxis    ,0>	translate < 0,mPosY,0> }
			object { mSolarWing2			rotate <0,90-mOffAxis+180,0>	translate < 0,mPosY,0> }
		}
	#else //for tests only
		#local mResult = union {
			object { mCan }
			object { mThrusterBox_b3		rotate <0,-90, 0>	translate < 0,-mHeight/2 + mThrusterSize/2, mRadius>	rotate <0,180,0> }
			object { mEngine				rotate <180,0, 0>	translate < 0,-mHeight/2, 0>	rotate <0,0,0> }
			object { mSolarWing				rotate <0,45,  0>	translate < 0,0,0> }
			object { mSolarWing				rotate <0,45+180,0>	translate < 0,0,0> }
		}
	#end
	
	// -------------------------------
	// Translate it
	// -------------------------------
	#if (mCenterType = "standing")
		#local mResult = object { mResult translate <0,mHeight/2 + TU_Private_LegsY_Max,0> }
	#elseif (mCenterType = "standing-real")
		#local mResult = object { mResult translate <0,mHeight/2 + TU_Private_LegsY_Real,0> }
	#elseif (mCenterType = "y+")
		#local hEngine	= MA_GetParam_F(mParams, "engine-height", 2);
		#local mResult = object { mResult translate <0,mHeight/2+hEngine,0> }
	#else //Default, return as assembled (centered body/cylinder)
	#end
	
	mResult
#end



#macro TU_PayloadAdapter (mDia, mHeight)

	#local hRing = 0.1;
	#local tRing = 0.1; //Thickness
	
	/*
	#local mRing = cylinder {
		<0,0,0> ,<0,hRing,0> ,mDia/2
		texture { pigment { color Gray80 } }
	}

	#local mRing = difference {
		object { mRing }
		object { mRing texture { pigment { color Gray10 } } scale <0.9, 2, 0.9>  }
		
	}
	*/
	#local mRing = difference {
		cylinder { <0,0,0> ,<0,hRing,0> ,mDia/2		texture { pigment { color Gray80 } } }
		cylinder { <0,-Nothing,0> ,<0,hRing+Nothing,0> ,mDia/2-tRing		texture { pigment { color Gray10 } } }
	}
	
	#local mResult = union {
		object { mRing }
		object { mRing translate <0,mHeight - 1*hRing, 0> }
		#for (mNum, 0, 3, 1)
			//cylinder { <0,0,0> ,<0,mHeight,0> tRing/2		texture { pigment { color Silver } } translate <mDia/2-tRing/2, 0, 0> rotate <0, mNum*90+45, 0> }
			cylinder { <0,0,0> ,<0,mHeight,0> tRing/2		texture { Chrome_Metal } translate <mDia/2-tRing/2, 0, 0> rotate <0, mNum*90+45, 0> }
		#end
	}
	
	mResult


#end


#macro TU_PayloadAdapter_old (mDia, mHeight)

	#local mResult = cylinder {
		<0,0,0> ,<0,mHeight,0> ,mDia/2
		texture { pigment { color Gray80 } }
	}

	#local mResult = difference {
		object { mResult }
		object { mResult texture { pigment { color Gray10 } } translate <0,-0.01, 0> scale <0.9, 1.02, 0.9> }
		
	}
	mResult


#end

// ****************************************************************************************************************************
//
// Booster
//
// ****************************************************************************************************************************
#macro TU_Booster	( mParentName, mNewName, mParams )
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);	#end

	#local mRadius			= MA_GetParam_F(mParams, "booster-radius",			0.3	);
	#local mHeight			= MA_GetParam_F(mParams, "booster-height",			4	);
	#local mHeightStrips	= MA_GetParam_F(mParams, "booster-height-strips",	0.1	);
	#local mNumStrips		= MA_GetParam_F(mParams, "booster-num-strips",		4	);
	#local mLengthCons		= MA_GetParam_F(mParams, "booster-length-cons",		0.3	);
	#local mRadiusCons		= MA_GetParam_F(mParams, "booster-height-cons",		0.05);
	
	#local mBoosterActive	= MA_GetParam_F(mParams, "booster-active",		0);
	#local mFlameHeight		= MA_GetParam_F(mParams, "booster-flame-height",0) * 2; //visible will only the upper half of the flame
	#local mFlameRadius		= MA_GetParam_F(mParams, "booster-flame-radius",0);

	
	#local mHeightCone		= mRadius;
	#local mHeightNozzle	= mRadius;
	#local mHeightCenter	= mHeight - mHeightCone - mHeightNozzle;
	
	
	#local mSeparationSecs	= -99;
	#ifdef( MA_Screenplay_Temp )
		#local mSeparationSecs	= SCR_GetNumber (mMyName, "", "separation",	mSeparationSecs);
		#local mBoosterActive	= SCR_GetNumber (mMyName, "", "active",		mBoosterActive);

		//http://www.povray.org/documentation/3.7.0/r3_3.html#r3_3_1_9_4
		//https://www.php.net/manual/en/function.strftime.php
		#local mSeed = seed (val(datetime(now, "%S")));
	#else
		#local mSeed = seed (val(datetime(now, "%S")));
	#end


	//-10 to 0 => Engine-flame gets smaller
	//0 to x => moves away to the sides, linear
	//1 to x => rotates
	//1 to x => starts moving downwards, slope bounce (increasing and increasing)
	//#local mSeparationSecs	= 1;
	#local mTranslate		= <0,0,0>;
	#local mRotate			= <0,0,0>;
	#if (mSeparationSecs > -10 & mSeparationSecs < 0)
		//Lower Flame
		#local mFlameRadius = -mSeparationSecs/10 * mFlameRadius*0.7  + mFlameRadius*0.3;
	#end
	#if (mSeparationSecs >= 0)
		//Move to the side and lower it
		//#local mTranslate = < -mSeparationSecs*2, -mSeparationSecs*0.5,0>;
		#local mTranslate = < -mSeparationSecs, -mSeparationSecs,0>;
		#local mFlameRadius = mFlameRadius/3;
	#end
	#if (mSeparationSecs >= 0)
		//Additionally start rotation
		#local mRotate = <0,0, mSeparationSecs*10>;
	#end
	
	//Stick together the Booster
	#local mResult = union {
		#local mH = 0;

		//The nozzle at first
		cone {
			<0, mH, 0> mRadius
			<0, mH + mHeightNozzle, 0> mRadius/10 open
			texture { TU_Booster_Nozzle_Texture }
		}

		//Center
		#local mH = mH + mHeightCone;
		cylinder { <0,mH,0> ,<0,mH + mHeightCenter,0> ,mRadius
			texture { pigment { color TU_Booster_ColorMain } }
		}
		//Cone/tip
		#local mH = mH + mHeightCenter;
		cone {
			<0, mH, 0> mRadius
			<0, mH + mHeightCone, 0> 0
			texture {	pigment { color TU_Booster_ColorMain }			}
		}
		//Strips
		#for (mX, 0, mNumStrips - 1)
			#local mH = mHeightCone + (  mHeightCenter - mHeightStrips) / (mNumStrips-1) * mX;
			cylinder { <0,mH,0> ,<0,mH + mHeightStrips,0> ,mRadius + Nothing
				texture { pigment { color TU_Booster_ColorStrips } }
			}
		#end
		//Connectors between Booster and Body
		#local mH = mHeightCone + mHeightCenter / 4 * 1;
		cylinder { <mRadius,mH,0> ,<mRadius + mLengthCons,mH,0> ,mRadiusCons
			texture { pigment { color TU_Booster_ColorCons } }
		}
		#local mH = mHeightCone + mHeightCenter / 4 * 3;
		cylinder { <mRadius,mH,0> ,<mRadius + mLengthCons,mH,0> ,mRadiusCons
			texture { pigment { color TU_Booster_ColorCons } }
		}
		//Flame
		#if (mBoosterActive > 0)

			difference	{
				sphere		{
					<0,-mRadius*0.5,0>,mRadius*mFlameRadius
					texture { TU_Booster_Flame_Texture }
				}
				box			{ <-mRadius*3,0,-mRadius*3> , <+mRadius*3,+mRadius*3,+mRadius*3> }
				translate	<0,-mRadius*0.02,0>
				scale		<1,mFlameHeight + mFlameHeight/5 * (rand(mSeed)-0.5) ,1>
			}
			
		#end

		
	}
	//#local mResult = object { mResult rotate y*180 translate <mRadius + mLengthCons, 0, 0> }
	#local mResult = object { mResult rotate mRotate translate mTranslate }
		
	mResult

#end



// ****************************************************************************************************************************
//
// Landinglegs and feet
//
// ****************************************************************************************************************************
#macro TU_LandingLegs	( mParentName, mNewName, mParams )
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);	#end
	
	#local mLength			= MA_GetParam_F(mParams, "legs-length",				2	);
	#local mLengthFactor	= MA_GetParam_F(mParams, "legs-length-factor",		1	);
	#local mLengthFactorMin	= MA_GetParam_F(mParams, "legs-length-factor-min",	0.2	);
	#local mOffsetLegs		= MA_GetParam_F(mParams, "legs-offset-legs",		1.0	);
	#local mOffsetFeet		= MA_GetParam_F(mParams, "legs-offset-feet",		2.5	);
	#local mRadiusLegs		= MA_GetParam_F(mParams, "legs-radius-legs",		0.02);
	#local mRadiusFeet		= MA_GetParam_F(mParams, "legs-radius-feet",		0.6	);
	#local mHeightFeet		= MA_GetParam_F(mParams, "legs-height-feet",		0.1	);
	
	
	#ifdef( MA_Screenplay_Temp )
		#local mLengthFactor	= SCR_GetNumber (mMyName, "", "stretched", mLengthFactor);
	#end
	
	//#local mLengthFactor	= 0; //for tests only
	
	#if (mLengthFactor < mLengthFactorMin) #local mLengthFactor	= mLengthFactorMin; 	#end
	
	
	#declare TU_Private_LegsY_Real	= mLength*mLengthFactor;
	#declare TU_Private_LegsY_Max	= mLength;
	
	
	#local mResult = union {
		//Stick/leg
		cylinder { <0,-0.5,0> ,<mOffsetFeet*mLengthFactor,(mLength-mHeightFeet)*mLengthFactor,0> ,mRadiusLegs
			texture { pigment { color TU_Legs_Color } }
			scale 1.05
		}
		//foot
		cylinder {
			<mOffsetFeet*mLengthFactor,(mLength*mLengthFactor)-mHeightFeet,0>,
			<mOffsetFeet*mLengthFactor,mLength*mLengthFactor,0>,
			mRadiusFeet
			texture { pigment { color TU_Feet_Color } }
		}
	}

	//If the foot (the plate) is very slim and the sticks are spread to the outside,
	//maybe the tip of the cylinder looks through the feet.
	//This deletes everything below the feet.
	#local mResult = difference {
		object { mResult }
		cylinder {
			<mOffsetFeet*mLengthFactor,mLength*mLengthFactor,0>,
			<mOffsetFeet*mLengthFactor,mLength*mLengthFactor+0.5,0>,
			mRadiusFeet
			texture { pigment { color TU_Feet_Color } }
		}
	}
	//Was build upwards, rotate the foot to the bottom
	#local mResult = object { mResult translate <mOffsetLegs, 0, 0> rotate <180,0,0> }
	mResult

#end


// ****************************************************************************************************************************
//
// Thruster 
//
// Size will always be <1,1,2> (doubled height because box and nozzle are both <1,1,1>)
//
// ****************************************************************************************************************************
#macro TU_Thruster ( mParentName, mNewName, mParams )
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);	#end
	
	#local mThrusterThrusting = 0;
	//#local mThrusterThrusting = 1;	//for tests only

	#local mFlameDist		= MA_GetParam_F(mParams, "thruster-flame-dist", 3);
	#local mFlameLength		= MA_GetParam_F(mParams, "thruster-flame-length", 20);
	#local mFlameDia1		= MA_GetParam_F(mParams, "thruster-dia-1", 0.8);
	#local mFlameDia2		= MA_GetParam_F(mParams, "thruster-dia-2", 3.5);

	/*
	#if (frame_number / 2 = int(frame_number / 2))
		//Allow thrusters to fire at round frame_numbers only
		#ifdef( MA_Screenplay_Temp )
			#local mThrusterThrusting = SCR_GetNumber (mMyName, "", "active", 0);
		#end
	#end
	*/
	//#local mThrusterThrusting = SCR_GetNumber (mMyName, "", "active", 0, 0);
	#ifdef( MA_Screenplay_Temp )
		#if (frame_number / 2 = int(frame_number / 2)) //pulsed thruster, fires only in every second frame
			#local mThrusterThrusting = SCR_GetNumber (mMyName, "", "active", 0);
		#end
	#end
	
	
	
	#local mResult = union {
		//Box behind the nozzle
		box {
			<0,0,0> <1,1,1> 
			texture {
				pigment{ color TU_Thruster_Box_Color}
				finish { reflection 0.1 phong 1}
			}
		}
		//Nozzle
		cone {
			<1, 0.5, 0.5> 0.2
			<2, 0.5, 0.5> 0.5 open
			texture {
				pigment { color TU_Thruster_Nozzle_Color }
				finish { reflection 0.1 phong 1}
			}
		}
		//Flame (if thruster is activated)
		#if (mThrusterThrusting > 0)
			cone {
				<mFlameDist, 0.5, 0.5>	mFlameDia1
				<mFlameDist + mFlameLength, 0.5, 0.5>	mFlameDia2
				texture { TU_Thruster_Flame_Texture	}
				no_shadow
			}
		#end
		translate <-1, 0, -0.5> 		//center all axes
	}
	
	mResult
#end




// ****************************************************************************************************************************
//
// ThrusterBox (Attitude-Control, chemical)
//
// SizeX = the length of the Box (0 = original Size of 5)
//
// ****************************************************************************************************************************

#macro TU_ThrusterBoxFor    ( mParentName, mNewName, mParams )
	object { TU_ThrusterBox ( mParentName, mNewName, mParams ) rotate <0, 90,-90> }
#end
#macro TU_ThrusterBoxBack   ( mParentName, mNewName, mParams )
	object { TU_ThrusterBox ( mParentName, mNewName, mParams ) rotate <0,-90,-90> }
#end

#macro TU_ThrusterBox (mParentName, mNewName, mParams )
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);	#end

	#local mThrusterSize = MA_GetParam_F(mParams, "thruster-size", 0.5);

	//Dont change them, use thruster-size instead
	#local mThrusterX = 2;	//Height/length of a nozzle/box-pair
	#local mBoxX = 5;
	#local mBoxY = 1;
	#local mBoxZ = 4;

	#local mResult = union {
		box {
			<0,0,0>
			<mBoxX, mBoxY, mBoxZ> 
			texture { 	pigment { color TU_ThrusterBox_Plate_Color }	}
		}
		object { TU_Thruster (mMyName, "tr", mParams ) rotate <  0,  0,  0> translate <mBoxX-mThrusterX/2,1,2.5> }	//right
		object { TU_Thruster (mMyName, "ta", mParams ) rotate <  0, 90,  0> translate <mThrusterX*1.25,1,1> }		//away
		object { TU_Thruster (mMyName, "tl", mParams ) rotate <  0,180,  0> translate <mThrusterX/2,1,2.5> }		//left
		object { TU_Thruster (mMyName, "tu", mParams ) rotate <  0,  0, 90> translate <mThrusterX*1.5,2,3.5> }		//upward
		translate <-mBoxX/2, 0,-mBoxZ/2>		// Move Y into > 0 area
		scale mThrusterSize*(1/mBoxX)
	}
	mResult
#end


// ****************************************************************************************************************************
//
// Main-Engine at the aft end of the spacecraft
//
// ****************************************************************************************************************************

#macro TU_MainEngine ( mParentName, mNewName, mParams)
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);	#end
	
	#local mHeight			= MA_GetParam_F(mParams, "engine-height", 2);
	#local mDia				= MA_GetParam_F(mParams, "engine-dia", 1.5);
	#local mFlameHeightRaw	= MA_GetParam_F(mParams, "engine-flame-height", 3);
	#local mFlameDist		= MA_GetParam_F(mParams, "engine-flame-dist", 1);

	#local mHeightBox		= mHeight * 0.2;
	#local mHeightEngine	= mHeight - mHeightBox;
	#local mDiaBox			= mDia / 3;
	#local mFlameDia		= mDia;

	#local mEngineActive	= MA_GetParam_F(mParams, "engine-active", 0);
	#local mEngineTurb		= MA_Clock; //Turbulences in Engine-Flames
	#ifdef( MA_Screenplay_Temp )
		#local mEngineActive	= SCR_GetNumber (mMyName, "", "active", mEngineActive);
		#local mEngineTurb		= SCR_Second - abs(SCR_Second);
	#end

	#local mSeed1 = seed (val(datetime(now, "%S"))+7);
	#local mSeed2 = seed (val(datetime(now, "%S"))+2);

	
	#local mResult = union {
		box {
			<-mDiaBox/2,0,-mDiaBox/2> <mDiaBox/2,mHeightBox,mDiaBox/2> 
			texture { pigment { color TU_Engine_Box_Color }	}
		}
		cone {
			<0, mHeightBox, 0> mDia / 8 <0, mHeight, 0> mDia/2 open
			texture { TU_Engine_Nozzle_Texture }
		}
		//Flamme
		#if (mEngineActive > 0)
		
			#if (1)
				//Original (21.5.19), working but ugly
				#local mHeightFlame = mHeight * mFlameDist + mFlameHeightRaw + mFlameHeightRaw/5* (rand (mSeed1) -0.5);
				cone {
					<0, mHeight * mFlameDist, 0> mFlameDia/2 <0, mHeightFlame, 0> mFlameDia + mFlameDia/3 * (rand (mSeed2) -0.5)
					//<0, mHeight * mFlameDist, 0> mFlameDia/2 <0, mHeight * mFlameDist + 1, 0> mFlameDia + mFlameDia/3 * (rand (mSeed2) -0.5)
					texture { TU_Engine_Flame_Texture }
					//scale <1,1,1>
				}
				
			//Some unfruitfull experiments
			#else
				//#local mHeightFlame = mHeight * mFlameDist + mFlameHeightRaw + mFlameHeightRaw/5* (rand (mSeed1) -0.5);
				#local mHeightFlame = mFlameHeightRaw + mFlameHeightRaw/5* (rand (mSeed1) -0.5);
				#if (1)
					cone {
						<0, 0, 0> mFlameDia/2 <0, 1, 0> mFlameDia + mFlameDia/3 * (rand (mSeed2) -0.5)
						//<0, mHeight * mFlameDist, 0> mFlameDia/2 <0, mHeight * mFlameDist + 1, 0> mFlameDia + mFlameDia/3 * (rand (mSeed2) -0.5)
						texture { TU_Engine_Flame_Texture }
						scale <1,mHeightFlame,1>
						translate <0,mHeight * mFlameDist,0>
						//scale <1,1,1>
					}
				#else
					difference	{
						sphere		{
							//<0,0,0>, 0.5
							<0,0,0>, mHeightFlame
							
							texture { TU_Booster_Flame_Texture }
						}
						box			{ <-1,0,-1> <+1,-1,+1> }
						
						scale <1,mHeightFlame,1>
						translate <0,mHeight * mFlameDist,0>
					}
				#end
			#end
		#end
	}
	
	mResult
#end




// ****************************************************************************************************************************
//
// Solarwing
//
// ****************************************************************************************************************************
#macro TU_SolarWing ( mParentName, mNewName, mParams )
	#ifndef( MA_Screenplay_Temp )	#local mMyName = ""; #else #local mMyName = SCR_GetName(mParentName, mNewName);	#end

	#local mMastHeight		= MA_GetParam_F(mParams, "sc-mast-height",	1);
	#local mWingHeight		= MA_GetParam_F(mParams, "sc-wing-height",	3);
	#local mWingWidth		= MA_GetParam_F(mParams, "sc-wing-width",	2);
	#local mPanels			= MA_GetParam_F(mParams, "sc-panels",		3);

	#local mMastRadius		= mMastHeight / 30;
	#local mWingThickness	= mWingWidth / 30;
	#local mCellThickness	= mWingThickness / 5;
	#local mCellBorder		= mWingThickness * 0.6;		//Metallic border around the cells
	#local mHingeHeight		= mMastRadius*2;			//Space between two Cells
	#local mHingeWidth		= mHingeHeight*4;

	#if (mPanels < 2) 		#local mPanels = 1; #end
	
	//Hinge - Separator between the cells
	#local mHinge = cylinder {
			<0,0,0> ,<0,mHingeWidth,0> ,mHingeHeight/2
			texture { pigment { color TU_Solarcells_Hinge_Color } }
			translate <0,-mHingeWidth/2,0>
			rotate <0,0,-90>
			translate <0,mHingeHeight/2,0>
	}
	#local mResult = union {
		//Mast
		cylinder { <0,0,0> ,<0,mMastHeight,0> ,mMastRadius
			texture { pigment { color TU_Solarcells_Mast_Color } }
		}
		
		#local mWingHeightOne = (mWingHeight - mHingeHeight * (mPanels-1)) / mPanels;
		#for (n, 0, mPanels -1)
			#local mTemp = n * (mWingHeightOne + mHingeHeight);
			//Wing
			box {
				<-mWingWidth/2, mMastHeight + mTemp, -mWingThickness/2>
				< mWingWidth/2, mMastHeight + mTemp + mWingHeightOne, mWingThickness/2> 
				texture { pigment { color TU_Solarcells_Border_Color } }
			}
			//Solarcell (Thin layer atop wing)
			box {
				<-mWingWidth/2 + mCellBorder, mMastHeight + mTemp + mCellBorder, -mWingThickness/2-0.000001>
				< mWingWidth/2 - mCellBorder, mMastHeight + mTemp + mWingHeightOne - mCellBorder, +mWingThickness/2+0.000001> //Cells on both sides
				//< mWingWidth/2 - mCellBorder, mMastHeight + mTemp + mWingHeightOne - mCellBorder, -mWingThickness/2-mCellThickness> //Cells only on one side
				texture { TU_Solarcells_Cells_Texture }
			}
			//Hinge - Separator between the cells
			#if (n < mPanels-1)
				object { mHinge translate <0,mMastHeight + mTemp + mWingHeightOne,0> }
			#end
		#end
	}
	
	mResult
#end






//-----------------------------------------------------------------------------------------------------------------------------
//This is the end of the very first If (that checks if this file was included before)
//Never delete it, or else ...
	#version MA_Tussi_Temp;
#end
//-----------------------------------------------------------------------------------------------------------------------------
