
/* ****************************************************************************************************************************

--------------------------------------------------------
MA_Tussi_Examples
--------------------------------------------------------
Author:			Martin Seydler
Website:		www.maetes.com
Documentation:	https://www.maetes.com/en/povray-os/ma_tussi
Date:			November 2018 an later
Version:		1.0 (December 2024)
Copyright:		See above (author), below near any macro (orignal author and sources) and everywhere (www.povray.org)
License:		Use it in your projects as you like. A mention and a link to my website would be nice.


Helpful Command-Lines (change the path):
------------------------------------------

povray +Ima_tussi_examples.pov +Otest/ma_tussi.png +w600 +h600 -GR +A
povray +Ima_tussi_examples.pov +Otest/ma_tussi.png +w800 +h600 +GD +GF -GR +GS +GW +Q9 +A
povray +Ima_tussi_examples.pov +Otest/ma_tussi.png +w1200 +h900 +GD +GF -GR +GS +GW +Q9 +A
povray +Ima_tussi_examples.pov +Otest/ma_tussi.png +w1920 +h1080 +GD +GF -GR +GS +GW +Q9 +A



To open the resulting picture in a Browser (change the path):
-----------------------------------------------------------------
file:///home/pi/Projekte/povray/packages-ma/space/ma_tussi/


*/// **************************************************************************************************************************


//-----------------------------------------------
//Standard-Stuff
//-----------------------------------------------
#version 3.7;
#include "colors.inc"
#include "textures.inc"

global_settings { assumed_gamma 1.0 }


#macro MA_Default()
	#default {
		texture { pigment { color rgb <0.9,0.0,0.5> } }
		finish { ambient 0.1 * LigIntensity diffuse 0.99 }
	}
#end

//-----------------------------------------------------------------------------------------------------
//Try to include ma_helpers. My dev-folder has highest priority.
//-----------------------------------------------------------------------------------------------------
#local mH = array[2] {"../../misc/ma_helpers/ma_helpers.inc", "ma_helpers.inc" }
#for (mI, 0, dimension_size(mH,1) - 1) #if (file_exists(mH[mI])) #include mH[mI] #end #end #ifndef (MA_Helpers_Temp) #error "Cant find 'ma_helpers.inc'"; #end


//-----------------------------------------------
//Include Tussi
//-----------------------------------------------
#include "ma_tussi.inc"


//-----------------------------------------------
//Include Skyfakes for some nice backgrounds
//-----------------------------------------------
//#declare MA_Include_Talk = 1;		// 1 = sends some debug-messages to the terminal, 0 doesnt.
#local mFolderSkyfakes = MA_Include("ma_skyfakes.inc", "");
#if (mFolderSkyfakes != "-1")
	MA_Terminal("Skyfakes found")
	#include concat(mFolderSkyfakes, "ma_skyfakes_textures.inc")
#else
	MA_Terminal("Skyfakes NOT found")
#end

                      


//-----------------------------------------------
//Declare some Variables
//-----------------------------------------------
#declare LigVector			= <8,1,-10>*1000;
#declare CamLookAt			= <0,0,0>;
#declare CamLocation		= <0 , 5 ,-15>*1.5; //
#declare CamAngle			= 60;
//#declare CamType			= "ultra_wide_angle";


//-----------------------------------------------
//Choose an example.
//Only the last "x" gets generated
//(easier to switch from one to the other)
//-----------------------------------------------
#local xExample	= "simple";
#local xExample	= "pic1";
#local xExample	= "pic2";
#local Example	= "test";

#local Example	= "center-type";


//-----------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------
#if (xExample = "simple")

	#ifdef (MA_Skyfakes_Temp)
		sky_sphere { SF_BackgroundStars_Pigment(1.5,1) }
		object {
			SF_Horizon1 ( "", "", "", "l", <0,195,0>)
		}
	#end
	
	#local Params = "sc-angle:90;sc-tilt:-30;engine-active: 0;engine-flame-dist:1.3;";
	object {
		TU_Tussi ("", "", Params, "")
		rotate <45,0,0> //Side-view
	}
	
//-----------------------------------------------------------------------------------------------------------------------
// Example 1
//-----------------------------------------------------------------------------------------------------------------------
#elseif (xExample = "pic1")

	#ifdef (MA_Skyfakes_Temp)
		sky_sphere { SF_BackgroundStars_Pigment(1.5,1) }
	#end

	#local Params = "sc-angle:0;sc-tilt:0;engine-active: 1;engine-flame-dist:1.3";
	object {
		TU_Tussi ("", "", Params, "")
		scale 1
		//rotate <65,80,0> //side-view
		rotate <45,0,0> //side-view
	}

//-----------------------------------------------------------------------------------------------------------------------
// Example 2
//-----------------------------------------------------------------------------------------------------------------------
#elseif (xExample = "pic2")

	#declare LigVector			= <-10,20,-10>*100;

	#ifdef (MA_Skyfakes_Temp)
		sky_sphere { SF_BackgroundStars_Pigment(1.5,1) }
	#end
	
	#local Params = "sc-angle:90;sc-tilt:-75;sc-wing-height:6;sc-wing-width:2;sc-panels:5;engine-height:1.0;body-dia:3; thruster-size:0.6";
	#declare TU_Spacecraft_Plates_Texture =	texture {
		pigment { color <0.4,0.2,0.2>*2.0 }
		normal	{ bumps 0.1 scale 0.010 }
		finish { reflection 1 phong 1}
	}

	#declare TU_Spacecraft_Texture = texture {
		pigment { color rgb <0.9, 0.9, 0.9> }
		//normal	{ bumps 0.2 scale 0.5 }	//Größeres Bumps = höhere Schatten. Scale x+z-Größe der Schatten
		normal	{ bumps 0.7 scale 0.1 }	//Gekrissel (Sandgestrahlt? statt sowas wie Hitzeschutz-Folien
		finish	{ ambient 0.1 phong 1}	//Ambient = Schattenhelligkeit (Leider aller Schatten, nicht nur der durch Normal erzeugten)
	}
	#declare TU_Thruster_Box_Color		= color <1,1,1>/9;
	#declare TU_Thruster_Nozzle_Color	= color <3,3,3>/9;
	#declare TU_ThrusterBox_Plate_Color	= color White;
	#declare TU_Engine_Box_Color		= color <5,5,5>/9;

	
	object {
		TU_Tussi ("", "", Params, "")
		scale 1.2
		rotate z*50
		rotate <30,90,0>
	}

//-----------------------------------------------------------------------------------------------------------------------
// Test
//-----------------------------------------------------------------------------------------------------------------------
#elseif (xExample = "test")

	

	#ifdef( MA_Skyfakes_Temp)
		sky_sphere {
			SF_BackgroundStars_Pigment(1.5,1)
		}
	#end

	#local Params = "sc-angle:0;sc-tilt:0;booster: 1; booster-active: 1;engine-active: 1;engine-flame-dist:1.3";
	object {
		TU_Tussi ("", "", Params, "")
		scale 1
		//rotate <65,80,0> //side-view
		//rotate <45,+0,0> //side-below
		
	}
	
	#declare CamLocation = CamLocation + <0,0,-0>;


//-----------------------------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------------
#elseif (xExample = "center-type")

	//MA_CenterMarker ( 10 )
	MA_Testarea_Base ( 10, 10, 4, 4 )
	#local Params = "sc-angle:0;sc-tilt:0;engine-active: 1; legs-length-factor: 1";
	object {
		TU_Tussi ("", "", Params, "standing")	//all hardware in Y+
		scale 1
		translate <-5,0,0>
		//rotate <65,80,0> //side-view
		//rotate <45,+0,0> //side-below
		
	}
	object {
		TU_Tussi ("", "", Params, "")	//Centered body, ideal to rotate it.
		scale 1
		translate <+5,0,0>
		//rotate <65,80,0> //side-view
		//rotate <45,+0,0> //side-below
		
	}

//-----------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------
#else
	//Warning, no existing example choosen
	#error "*** no existing example choosen ***"
#end




MA_CameraAndLight_Set()




